import numpy as np
from sklearn.metrics import mean_squared_error, r2_score, accuracy_score, f1_score, confusion_matrix, \
    classification_report


def classifier_outputs(y_test, y_pred, f1_average='micro'):
    accuracy = round(accuracy_score(y_test, y_pred), 3)
    f1 = round(f1_score(y_test, y_pred, average=f1_average), 3)
    conf_mat = confusion_matrix(y_test, y_pred)
    report = classification_report(y_test, y_pred)
    final = f'''The accuracy is: {accuracy}.\nThe f1-score on average = {f1_average} is: {f1}.\n
The confusion matrix:\n{conf_mat}\n
Full classification report:\n{report}'''
    return final


def regression_outputs(y_test, y_pred):
    mape = round(mean_squared_error(y_test, y_pred), 3)
    rmse = round(np.sqrt(mean_squared_error(y_test, y_pred)), 3)
    r2 = round(r2_score(y_test, y_pred), 3)
    final = f'''The mape is: {mape}.\nThe rmse is: {rmse}.\nThe r2 score is: {r2}\n'''
    return final



import numpy as np
import pandas as pd
import warnings
from sklearn.naive_bayes import GaussianNB
from sklearn.naive_bayes import MultinomialNB
from sklearn.naive_bayes import BernoulliNB
from sklearn.linear_model import SGDClassifier, PassiveAggressiveClassifier, RidgeClassifier, \
    LinearRegression, RANSACRegressor, ARDRegression, HuberRegressor, LogisticRegression, \
    LogisticRegressionCV, SGDRegressor, TheilSenRegressor, PassiveAggressiveRegressor, Lasso, Ridge
from sklearn.neural_network import MLPClassifier, MLPRegressor
from sklearn.ensemble import AdaBoostClassifier, BaggingClassifier, RandomForestClassifier, \
    RandomForestRegressor, AdaBoostRegressor, BaggingRegressor
from sklearn import svm
# from xgboost import XGBClassifier, XGBRegressor
from sklearn.discriminant_analysis import QuadraticDiscriminantAnalysis, LinearDiscriminantAnalysis
from sklearn.tree import DecisionTreeClassifier, ExtraTreeClassifier, DecisionTreeRegressor, \
    ExtraTreeRegressor
from sklearn.neighbors import KNeighborsClassifier, KNeighborsRegressor
# from lightgbm import LGBMClassifier, LGBMRegressor

warnings.filterwarnings("ignore", category=FutureWarning)

np.random.seed(3)


# Specific Function to predict the output
def predicting(model, data, labels, test_data):
    final_model = model
    y_pred = final_model.fit(data, labels).predict(test_data)
    output = pd.DataFrame()
    output['predictions'] = y_pred
    return output


# ------------------------------------------CLASSIFICATION---------------------------------------------------

# General function for random forest classification
def random_forest_classifier(data, label, test):
    model = RandomForestClassifier()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for decision tree classification
def decision_tree_classifier(data, label, test):
    model = DecisionTreeClassifier()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for support vector machine classification
def svm_classifier(data, label, test):
    model = svm.SVC()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for extra-tree classification
def extra_tree_classifier(data, label, test):
    model = ExtraTreeClassifier()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for gaussian classification
def gaussian_classifier(data, label, test):
    model = GaussianNB()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for logistic-regression i.e. classification
def logistic_classifier(data, label, test):
    model = LogisticRegression()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for logistic-regression with cross validation i.e. classification
def logistic_cv_classifier(data, label, test):
    model = LogisticRegressionCV()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for bernoulli classification
def bernoulli_classifier(data, label, test):
    model = BernoulliNB()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for multinomial classification
def multinomial_classifier(data, label, test):
    model = MultinomialNB()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for stochastic gradient descent classification
def sgd_classifier(data, label, test):
    model = SGDClassifier()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for passive aggressive classification
def passive_aggressive_classifier(data, label, test):
    model = PassiveAggressiveClassifier()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for ridge classification
def ridge_classifier(data, label, test):
    model = RidgeClassifier()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for multi-layer-perceptron classification
def mlp_classifier(data, label, test):
    model = MLPClassifier()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for adaboost classification
def adaboost_classifier(data, label, test):
    model = AdaBoostClassifier()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for bagging classification
def bagging_classifier(data, label, test):
    model = BaggingClassifier()
    ypred = predicting(model, data, label, test)
    return ypred


# # General function for xgboost classification
# def xgboost_classifier(data, label, test):
#     model = XGBClassifier()
#     ypred = predicting(model, data, label, test)
#     return ypred


# # General function for light-GBM classification
# def light_gbm_classifier(data, label, test):
#     model = LGBMClassifier()
#     ypred = predicting(model, data, label, test)
#     return ypred


# General function for linear-discriminant-analysis classification
def lda_classifier(data, label, test):
    model = LinearDiscriminantAnalysis()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for quadratic-discriminant-analysis classification
def qda_classifier(data, label, test):
    model = QuadraticDiscriminantAnalysis()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for k-nearest-neighbour classification
def knn_classifier(data, label, test):
    model = KNeighborsClassifier()
    ypred = predicting(model, data, label, test)
    return ypred


# ------------------------------------------REGRESSION---------------------------------------------------


# General function for k-nearest-neighbour regression
def knn_regressor(data, label, test):
    model = KNeighborsRegressor()
    ypred = predicting(model, data, label, test)
    return ypred


# # General function for light-GBM regression
# def light_gbm_regressor(data, label, test):
#     model = LGBMRegressor()
#     ypred = predicting(model, data, label, test)
#     return ypred


# # General function for xgboost regression
# def xgboost_regressor(data, label, test):
#     model = XGBRegressor()
#     ypred = predicting(model, data, label, test)
#     return ypred


# General function for linear regression
def linear_regressor(data, label, test):
    model = LinearRegression()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for ransac regression
def ransac_regressor(data, label, test):
    model = RANSACRegressor()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for ARD regression
def ARD_regressor(data, label, test):
    model = ARDRegression()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for huber regression
def huber_regressor(data, label, test):
    model = HuberRegressor()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for stochastic-gradient-descent regression
def sgd_regressor(data, label, test):
    model = SGDRegressor()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for theilsen regression
def theilsen_regressor(data, label, test):
    model = TheilSenRegressor()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for passive aggressive regression
def passive_aggressive_regressor(data, label, test):
    model = PassiveAggressiveRegressor()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for multi-layered-perceptron regression
def mlp_regressor(data, label, test):
    model = MLPRegressor()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for adaboost regression
def adaboost_regressor(data, label, test):
    model = AdaBoostRegressor()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for random_forest regression
def random_forest_regressor(data, label, test):
    model = RandomForestRegressor()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for decision-treee regression
def decision_tree_regressor(data, label, test):
    model = DecisionTreeRegressor()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for support vector machine regression
def svm_regressor(data, label, test):
    model = svm.SVR()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for bagging regression
def bagging_regressor(data, label, test):
    model = BaggingRegressor()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for extra-tree regression
def extra_tree_regressor(data, label, test):
    model = ExtraTreeRegressor()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for Lasso regression
def lasso_regressor(data, label, test):
    model = Lasso()
    ypred = predicting(model, data, label, test)
    return ypred


# General function for Ridge regression
def ridge_regressor(data, label, test):
    model = Ridge()
    ypred = predicting(model, data, label, test)
    return ypred



import pandas as pd
import re
import numpy as np
from sklearn.preprocessing import LabelEncoder, StandardScaler, MinMaxScaler
from sklearn.model_selection import train_test_split

np.random.seed(7)


# Overview:
# This is the code for RiceBowl Preprocessing.
# There are various data pre-processing
# and cleaning functionalities.

# General function to read a csv file
def read_csv(path):
    data = pd.read_csv(path)
    return data


# General function to read an excel file
def read_excel(path, sheet_name):
    xl = pd.ExcelFile(path)
    df = pd.read_excel(xl, sheet_name)
    return df


# General function for formatting the column headers to lower case.
# All "spaces" and "-" are replaced by "_"
def reformat_col_headers(df):
    col_list = list(df.columns.values)
    processed_col_list = [re.sub(r'\s+', '_', item.strip().lower()) for item in
                          col_list]
    processed_col_list = [x.replace("-", "_") for x in processed_col_list]
    col_dict = dict(zip(col_list, processed_col_list))
    df.rename(columns=col_dict, inplace=True)
    return df


# General Function to convert string columns in date format to datetime
def str_to_datetime(df, **col_names):
    for i in col_names.values():
        df[i] = pd.to_datetime(df[i], dayfirst=True)
    return df


# General Function to convert timestamp columns to datetime
def timestamp_to_datetime(df, **col_names):
    for i in col_names.values():
        df[i] = pd.to_datetime(df[i], dayfirst=True, unit='s',
                               errors='coerce')
    return df


# General Function to convert datetime/str columns in datetime format to timestamp
def to_timestamp(df, **col_names):
    for col in col_names.values():
        df[col] = pd.to_datetime(df[col], utc=True, dayfirst=True)
        df[col] = df[col].apply(lambda x: x.timestamp())
    return df


# General Function to label encode the categorical columns
def label_encode(df, **col_names):
    le = LabelEncoder()
    for i in col_names.values():
        df[i] = le.fit_transform(df[i])
    return df, le


# General Function to one-hot encode the categorical columns
def one_hot_encode(df, **col_names):
    for col in col_names.values():
        encoded_column = pd.get_dummies(df[col], prefix=col)
        df = pd.concat([df, encoded_column], axis=1).drop(col, axis=1)

    return df


# General Function to calculate the difference between 2 date columns.
def dates_diff(df, col1, col2, diff_type='days'):
    df = str_to_datetime(df, c1=col1, c2=col2)
    if diff_type == 'days':
        df['days'] = (np.absolute(df[col1] - df[col2])).dt.days
    elif diff_type == 'months':
        df['months'] = (np.absolute(df[col1] - df[col2])).dt.days // 30
    elif diff_type == 'years':
        df['years'] = (np.absolute(df[col1] - df[col2])).dt.days // 365
    elif diff_type == 'weeks':
        df['weeks'] = (np.absolute(df[col1] - df[col2])).dt.days // 7
    else:
        print(
            'Wrong difference string provided. Please provide "days","weeks","years" or "months".')
    return df


# General Function to remove duplicate rows.
def drop_duplicates(df):
    df = df.drop_duplicates(ignore_index=True)
    return df


# General Function to reset index
def reset_index(df, drop=True):
    df = df.reset_index(drop=drop)
    return df


# General Function to convert a column to a particular datatype
def to_dtype(df, dtype, **cols):
    if dtype == str:
        for col in cols.values():
            df[col] = df[col].fillna(' ')
    for col in cols.values():
        df[col] = df[col].astype(dtype)

    return df


# General Function to fill null values with mode
def fill_mode(df, **col_names):
    for i in col_names.values():
        df[i] = df[i].fillna(df[i].mode()[0])
    return df


# General Function to fill null values with mean
def fill_mean(df, **col_names):
    for i in col_names.values():
        df[i] = df[i].fillna(round(df[i].mean(), 3))
    return df


# General Function to melt data
# parameters:
# df: DataFrame to be melted
# cols_to_melt: cols that are needed as rows instead of columns in the form of a list
# new_col_name: name of the new column formed by melted values in the form of a string
# value_name: (optional, default: 'value') name of value column of melted values
def melt(df, cols_to_melt, new_col_name, value_name='value'):
    unchanged_cols = list(set(df.columns) - set(cols_to_melt))
    new_df = pd.melt(
        df,
        id_vars=unchanged_cols, value_vars=list(cols_to_melt), var_name=new_col_name, value_name=value_name)

    return new_df


# General Function to make existing data a list of split values
def split_columns(df, orig_col, separator):
    df[orig_col] = df[orig_col].apply(lambda x: str(x).split(separator))
    return df


# General Function to remove unwanted characters from data
def remove_unwanted_chars(df, **col_names):
    for i in col_names.values():
        df[i] = df[i].map(lambda x: str(x).lstrip('*+-~$€£inr¥₹')
                          .rstrip('inrsec*sdmywminmokKmMbB'))
    return df


# General Function to fill "million M", "billion B", "thousand k", "lakhs L", "crore cr"
def fill_num_abbreviations(df, **col_names):
    for i in col_names.values():
        df = to_dtype(df, str, c1=i)
        df[i] = df[i].apply(lambda x: x.lower())
        df[i] = df[i].apply(lambda x: x + '000000000' if 'b' in str(x) else x).apply(
            lambda x: x.replace('b', '') if 'b' in str(x) else x)
        df[i] = df[i].apply(lambda x: x + '000000' if 'm' in str(x) else x).apply(
            lambda x: x.replace('m', '') if 'm' in str(x) else x)
        df[i] = df[i].apply(lambda x: x + '000' if 'k' in str(x) else x).apply(
            lambda x: x.replace('k', '') if 'k' in str(x) else x)
        df[i] = df[i].apply(lambda x: x + '00000' if 'l' in str(x) else x).apply(
            lambda x: x.replace('l', '') if 'l' in str(x) else x)
        df[i] = df[i].apply(lambda x: x + '0000000' if 'cr' in str(x) else x).apply(
            lambda x: x.replace('cr', '') if 'cr' in str(x) else x)
    return df


# General Function to split data for modeling purpose
def split_data(data, label, test_size=0.3):
    x_train, x_test, y_train, y_test = train_test_split(data, label, test_size=test_size,
                                                        random_state=7,
                                                        shuffle=True)
    y_train = np.array(y_train)
    y_test = np.array(y_test)
    return x_train, x_test, y_train, y_test


# General Function to find correlation excluding all null values
def find_corr(df, method='pearson'):
    corr = df.corr(method=method)
    return corr


# General Function to find outliers in a random variable using zscore
def zscore_outliers(series):
    data = list(series)
    mean = np.mean(data)
    std = np.std(data)
    threshold = 2.5
    outlier = []
    for i in data:
        z = (i - mean) / std
        if z > threshold:
            outlier.append(i)
    return outlier


# General function to standardize the data using standard scaler.
def standarization(data, list_of_cols=['ALL']):
    scaling = StandardScaler()
    if list_of_cols == ['ALL']:
        cols = list(data.columns)
    else:
        cols = list_of_cols

    data = scaling.fit_transform(data[cols])
    df = pd.DataFrame(data, columns=cols)
    return df


# General function to normalize the data using min-max scaler.
def normalization(data, list_of_cols=['ALL']):
    scaling = MinMaxScaler()
    if list_of_cols == ['ALL']:
        cols = list(data.columns)
    else:
        cols = list_of_cols

    data = scaling.fit_transform(data[cols])
    df = pd.DataFrame(data, columns=cols)
    return df


# General function to get all the basic stats of the data
def basic_stats(data, file=None):
    s = ''
    s = s + '------------Unique values:-----------------\n'
    s = s + str(data.nunique())
    s = s + '\n\n------------Null values:-----------------\n'
    s = s + str(data.isnull().sum())
    s = s + '\n\n------------Dtypes:-----------------\n'
    s = s + str(data.dtypes)
    temp = data.drop_duplicates()
    duplicates = data.shape[0] - temp.shape[0]
    s = s + '\n\n------------Duplicate values:-----------------\n'
    s = s + str(duplicates)
    if file == None:
        print(s)
    else:
        f = open(file, 'w')
        f.write(s)
        f.close()
        print('Written in file: {}'.format(file))



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("requirements.txt", "r") as fh:
    requirements = fh.read()

requirements = requirements.split('\n')

setuptools.setup(
    name="ricebowl",
    version="0.5.0",
    author="Shivek Chhabra",
    author_email="shivekchhabra@gmail.com",
    description="This package allows the users to preprocess dataframes and images, plot the data and then use ml models using a single command",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/shivekchhabra/ricebowl",
    packages=setuptools.find_packages(),
    install_requires=requirements,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



import seaborn as sns
import matplotlib.pyplot as plt
from ricebowl.ricebowl.processing import data_preproc


# General function to get a pair plot of the entire data. Hue can be modified.
def pairplot(df, hue=None, cols=['ALL']):
    if cols == ['ALL']:
        sns.pairplot(data=df, hue=hue)
    else:
        sns.pairplot(data=df, hue=hue, vars=cols)
    plt.show()


# General function to get plots for all columns passed. Press 'q' for next figure.
def distribution(df, **col_names):
    for i in col_names.values():
        sns.distplot(df[i], label=i)
        plt.show()


# General function to plot relationship between 2 random variables. x,y have input types as list/df series.
def plot(x, y, xlabel='x', ylabel='y'):
    plt.plot(x, 'g*', y, 'ro')
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)
    plt.legend([xlabel, ylabel])
    plt.show()


# General function to plot a scatterplot of the data
def scatter(data, x=None, y=None):
    sns.scatterplot(x=x, y=y, data=data)
    plt.show()


# General function to plot a boxplot of the data
def box(data):
    sns.boxplot(data=data)
    plt.show()


# General function for plotting a pie chart
def pie_chart(data, column_name, title='Title', labels=['None'], convert=False):
    if title == 'Title':
        title = column_name.capitalize()
    if convert == True:
        labels = list(data[column_name].unique())
        data, le = data_preproc.label_encode(data, c1=column_name)

    uniques = list(data[column_name].unique())
    if labels == ['None']:
        labels = labels * len(uniques)

    values = []
    for i in uniques:
        values = values + [data[column_name][data[column_name] == i].count()]
    fig1, ax1 = plt.subplots(figsize=(10, 8))
    ax1.pie(values, labels=labels, autopct='%1.1f%%',
            shadow=True, startangle=90)
    ax1.axis('equal')
    plt.title(title, size=20)
    plt.show()
